# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimDate,
    HprimForeign,
    HprimString,
    Segment,
)

logger = logging.getLogger("driverng.bd_max")


class BdMaxSegmentH(HeaderSegment):
    sender_name = HprimString(5)
    version_number = HprimString(13)
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class BdMaxSegmentP(Segment):
    patient_id = HprimString(4, max_length=20, trunc=True)

    orders = HprimForeign("O", "patient")


class BdMaxSegmentO(Segment):
    accession_number = HprimString((3, 1))
    test_id = HprimString((5, 4))
    test_name = HprimString((16, 1))

    results = HprimForeign("R", "order")


class BdMaxSegmentR(Segment):
    code = HprimString((3, 4))
    # INST_POSITIVE
    # INST_NEGATIVE
    # CONFIRMED_POSITIVE
    test_status = HprimString(4)
    # P: Preliminary
    # F: Final
    result_status = HprimString(9)
    date_result = HprimDate(12)
    instrument_type = HprimString((14, 1))
    machine_instrument_number = HprimString((14, 4))
    location_information = HprimString((14, 5))
    test_name = HprimString((14, 8))
    confirmed_positive = HprimString((15, 2))


class BdMaxSegmentQ(Segment):
    patient_id_request = HprimString((3, 1))
    accession_request = HprimString((3, 2))
    test_status = HprimString((5, 2))
    starting_date = HprimDate(7)
    end_date = HprimDate(8)
    request_info_status = HprimString(13)


class BdMaxSegmentL(Segment):
    terminaison = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {"code": "bd_max", "label": "BD MAX Becton Dickinson"}
    SEGMENT_CLASS_MAP = {
        "H": BdMaxSegmentH,
        "P": BdMaxSegmentP,
        "O": BdMaxSegmentO,
        "R": BdMaxSegmentR,
        "Q": BdMaxSegmentQ,
        "L": BdMaxSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessageV2.BooleanOption(
            "bd_max_envoi_prenom",
            label="Envoyer le prénom du patient dans le champ P.4 (attention "
            "champ limité à 20 caractères maximum)",
        ),
        AstmMessageV2.BooleanOption(
            "bd_max_tube_secondaire",
            label="Utilisation du tube secondaire au lieu du tube primaire (BDMAX connecté avec la KIESTRA)",
        ),
    ]

    def __init__(self, **kwargs):
        self.envoi_prenom = kwargs.pop("bd_max_envoi_prenom", False)
        self.tube_secondaire = kwargs.pop("bd_max_tube_secondaire", False)
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = BdMaxSegmentH(type_segment="H", **kwargs)
        message.end = BdMaxSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))
        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
        ):
            for sample, action in patient.samples:
                if self.tube_secondaire:
                    if self.envoi_prenom:
                        patient_id = "{} {}".format(
                            patient.last_name, patient.first_name
                        )
                    else:
                        patient_id = patient.last_name
                    logger.info(
                        "BD MAX sample_id : %s barcode_secondary %s media_code %s",
                        sample.sample_id,
                        sample.barcode_secondary_media,
                        sample.media_code,
                    )
                    liste_media_code = (
                        sample.media_code.split("_") if sample.media_code else ""
                    )
                    # Tube secondaire si existant sinon envoi du tube primaire
                    # Si le tube secondaire est composé du tube primaire, il ne
                    # faut pas l'envoyer au BD MAX car il s'agit d'un tube
                    # généré par la KIESTRA mais généré par MCA, dans ce cas,
                    # il faut envoyer le tube primaire
                    tube = sample.sample_id
                    for index, sample_secondary in enumerate(
                        sample.barcode_secondary_media
                    ):
                        logger.info(
                            "BD MAX Tube primaire %s tube secondaire %s boite %s",
                            tube,
                            sample_secondary,
                            liste_media_code[index],
                        )
                        if (
                            sample_secondary
                            and sample.sample_id in sample_secondary
                            and liste_media_code
                            and liste_media_code[index]
                        ):
                            continue
                        elif (
                            sample_secondary
                            and sample.sample_id not in sample_secondary
                            and liste_media_code
                            and liste_media_code[index]
                        ):
                            tube = sample_secondary
                        elif (
                            not sample_secondary
                            or not liste_media_code
                            or not liste_media_code[index]
                        ):
                            tube = sample.sample_id
                        else:
                            continue
                    seg_p = msg.patients.add(patient_id="{:.20}".format(patient_id))
                    for order in sample.orders:
                        seg_p.orders.add(
                            accession_number=tube,
                            test_id="BDMAX_AND",
                            test_name=order.code,
                        )
                else:
                    if self.envoi_prenom:
                        patient_id = "{} {}".format(
                            patient.last_name, patient.first_name
                        )
                    else:
                        patient_id = patient.last_name
                    seg_p = msg.patients.add(patient_id="{:.20}".format(patient_id))
                    for order in sample.orders:
                        seg_p.orders.add(
                            # Tube primaire
                            accession_number=sample.sample_id,
                            test_id="BDMAX_AND",
                            test_name=order.code,
                        )
        if msg.patients:
            return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient", sample_id=seg_q.accession_request.strip()
                )
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:
                tube = ""
                if self.tube_secondaire and seg_p.patient_id:
                    tube = seg_p.patient_id.split(" ")[0]
                for seg_o in seg_p.orders:
                    sample = worklist.get_or_create(
                        type="patient",
                        sample_id=tube if tube else seg_o.accession_number.strip(),
                    )
                    # Le BD MAX ne transmet pas le statut du résultat
                    for seg_r in seg_o.results:
                        sample.results.add(
                            code="%s%s" % (seg_r.code, seg_r.test_name),
                            value=seg_r.test_status,
                            date=seg_r.date_result,
                        )
        return worklist
